/*
** Module   : TableMapper.java
** Abstract : Table and field name to converted DMO and property name (bidirectional).
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20131015 Created initial version.
*/

package com.goldencode.p2j.persist;

import java.lang.reflect.*;
import java.util.*;

import com.goldencode.p2j.security.*;

/**
 * This class performs a bi-directional runtime mapping of DMO implementation classes (permanent 
 * or temporary) to their associated legacy table name. The mapping for a DMO implementation class
 * is held in a {@link LegacyTableInfo} instance.
 * <p>
 * For physical DBs, the mappings for the permanent DBs (connected at server startup) will be 
 * registered at server startup. For permanent DBs connected at application runtime, the mappings
 * will be registered and deregistered at the appropriate DB connection/disconnection times.
 * <p>
 * All the mappings for temporary tables will be registered during application lifetime, as the
 * temp tables get created or dropped.
 * 
 * @param    <K>
 *           The type of the key used by the {@link #p2j progress-to-java} map. For permanent
 *           DMOs, this will be a {@link String}. For temporary DMOs, this will be an 
 *           {@link Integer} value with the <code>UNIQUE-ID</code> identifying the 
 *           {@link TempTable} resource.
 */
public class TableMapper<K>
{
   /** Holds the mappers for all permanent DBs, by their schema name. */
   private static final Map<String, PermanentTableMapper> permanentDBs = 
      new HashMap<String, PermanentTableMapper>();

   /** Context-local mapper for all temporary tables. */
   private static final ContextLocal<TempTableMapper> tempDB = new ContextLocal<TempTableMapper>()
   {
      protected TempTableMapper initialValue()
      {
         return new TempTableMapper();
      };
   };
   
   /** Mapping of DMO implementation classes to their {@link LegacyTableInfo} instance. */
   protected final Map<Class<?>, LegacyTableInfo> j2p = new HashMap<Class<?>, LegacyTableInfo>();
   
   /**
    * Mapping of the legacy tables (identified by a custom key) to their {@link LegacyTableInfo} 
    * instance. */
   protected final Map<K, LegacyTableInfo> p2j = new HashMap<K, LegacyTableInfo>();

   /** The schema name. */
   private final String schema;
   
   /**
    * Create a new mapper for the given schema.
    * 
    * @param    schema
    *           The schema name.
    */
   private TableMapper(String schema)
   {
      this.schema = schema;
   }
   
   /**
    * Get the legacy table name for the DMO referenced by the given buffer.
    * 
    * @param    buffer
    *           A buffer instance.
    * 
    * @return   The legacy table name.
    */
   public static String getLegacyName(Buffer buffer)
   {
      RecordBuffer buf = ((BufferImpl) buffer).buffer();
      if (buf.isTemporary())
      {
         return getLegacyName((TempTable) buffer.tableHandle().get());
      }
      else
      {
         return getLegacyName(buf.getDMOImplementationClass());
      }
   }
   
   /**
    * Get the legacy field name for the property associated with the DMO referenced by the given
    * buffer.
    * 
    * @param    buffer
    *           A buffer instance.
    * @param    property
    *           The DMO property name.
    * 
    * @return   The legacy field name.
    */
   public static String getLegacyFieldName(Buffer buffer, String property)
   {
      RecordBuffer buf = ((BufferImpl) buffer).buffer();
      if (buf.isTemporary())
      {
         return getLegacyFieldName((TempTable) buffer.tableHandle().get(), property);
      }
      else
      {
         return getLegacyFieldName(buf.getDMOImplementationClass(), property);
      }
   }
   
   /**
    * Get the legacy table name for the given DMO, which is part of a permanent DB.
    * 
    * @param    dmoClass
    *           The DMO implementation class for a permanent DMO.
    *           
    * @return   The legacy table name.
    * 
    * @throws   IllegalStateException
    *           If a DMO belonging to the {@link DatabaseManager#TEMP_TABLE_SCHEMA} temp schema
    *           is passed as a parameter; for these cases, {@link #getLegacyName(TempTable)}
    *           or {@link #getLegacyName(Buffer)} must be used.
    */
   public static String getLegacyName(Class<?> dmoClass)
   {
      String schema = DatabaseManager.getSchemaByClass(dmoClass);
      if (DatabaseManager.TEMP_TABLE_SCHEMA.equals(schema))
      {
         final String msg =
            "Legacy table name for a temporary DMO can not be resolved using this API!";
         throw new IllegalArgumentException(msg);
      }

      synchronized (permanentDBs)
      {
         PermanentTableMapper mapper = locatePerm(schema);
         
         return mapper.legacyName(dmoClass);
      }
   }
   
   /**
    * Get the legacy field name for the given property, which is defined by a DMO part of a 
    * permanent DB.
    * 
    * @param    dmoClass
    *           The DMO implementation class for a permanent DMO.
    * @param    property
    *           The DMO property name.
    *           
    * @return   The legacy field name.
    * 
    * @throws   IllegalStateException
    *           If a DMO belonging to the {@link DatabaseManager#TEMP_TABLE_SCHEMA} temp schema
    *           is passed as a parameter; for these cases, {@link #getLegacyFieldName(TempTable, 
    *           String)} or {@link #getLegacyFieldName(Buffer, String)} must be used.
    */
   public static String getLegacyFieldName(Class<?> dmoClass, String property)
   {
      String schema = DatabaseManager.getSchemaByClass(dmoClass);
      if (DatabaseManager.TEMP_TABLE_SCHEMA.equals(schema))
      {
         final String msg =
            "Legacy field names for a property of a temporary DMO can not be resolved using " +
            "this API!";
         throw new IllegalArgumentException(msg);
      }

      synchronized (permanentDBs)
      {
         PermanentTableMapper mapper = locatePerm(schema);
         
         return mapper.legacyFieldName(dmoClass, property);
      }
   }
   
   /**
    * Get the legacy table name for the given {@link TempTable} resource.
    * 
    * @param    tt
    *           The {@link TempTable} resource.
    *           
    * @return   The legacy table name.
    */
   public static String getLegacyName(TempTable tt)
   {
      TempTableMapper mapper = locateTemp();
      Class<?> dmoClass = getDMOClass(tt);
      
      return mapper.legacyName(dmoClass);
   }
   
   /**
    * Get the legacy field name for the given {@link TempTable} resource and property.
    * 
    * @param    tt
    *           The {@link TempTable} resource.
    * @param    property
    *           The DMO property name.
    * 
    * @return   The legacy field name.
    */
   public static String getLegacyFieldName(TempTable tt, String property)
   {
      TempTableMapper mapper = locateTemp();
      Class<?> dmoClass = getDMOClass(tt);

      return mapper.legacyFieldName(dmoClass, property);
   }

   /**
    * Get the DMO implementation class for the specified {@link TempTable} resource.
    * 
    * @param    table
    *           The {@link TempTable} resource.
    *           
    * @return   The DMO implementation class.
    */
   public static Class<?> getDMOClass(TempTable table)
   {
      // no reason to search the mappings, get it from the persistence info
      TempTableBuilder ttb = (TempTableBuilder) table;
      return ttb.getPersistentClass().getMappedClass();
   }

   /**
    * Resolve the DMO property name for a legacy field name belonging to the specified 
    * {@link TempTable} resource.
    * 
    * @param    table
    *           The {@link TempTable} resource.
    * @param    field
    *           The legacy field name.
    *           
    * @return   The DMO property name.
    */
   public static String getPropertyName(TempTable table, String field)
   {
      TempTableMapper mapper = locateTemp();
      return mapper.getPropertyName(table.getUniqueID().toJavaIntegerType(), field);
   }
   
   /**
    * Get the DMO implementation class for the legacy table name (part of the specified schema).
    * 
    * @param    schema
    *           The schema name.
    * @param    table
    *           The legacy table name.
    *           
    * @return   The DMO implementation class.
    * 
    * @throws   IllegalArgumentException
    *           If a table belonging to the {@value DatabaseManager#TEMP_TABLE_SCHEMA} is used.
    */
   public static Class<?> getDMOClass(String schema, String table)
   {
      if (DatabaseManager.TEMP_TABLE_SCHEMA.equals(schema))
      {
         final String msg = "DMO for a temporary table can not be resolved using this API!";
         throw new IllegalArgumentException(msg);
      }

      PermanentTableMapper mapper = locatePerm(schema);
      return mapper.getDMOClass(table.toLowerCase());
   }
   
   /**
    * Get the DMO property name for a legacy field name belonging to the specified legacy table.
    * 
    * @param    schema
    *           The schema name.
    * @param    table
    *           The legacy table name.
    * @param    field
    *           The legacy field name.
    *           
    * @return   The DMO property name.
    * 
    * @throws   IllegalArgumentException
    *           If a table belonging to the {@value DatabaseManager#TEMP_TABLE_SCHEMA} is used.
    */
   public static String getPropertyName(String schema, String table, String field)
   {
      if (DatabaseManager.TEMP_TABLE_SCHEMA.equals(schema))
      {
         final String msg = "Properties for temporary DMOs can not be resolved using this API!";
         throw new IllegalArgumentException(msg);
      }

      PermanentTableMapper mapper = locatePerm(schema);
      return mapper.getPropertyName(table.toLowerCase(), field);
   }

   /**
    * Add the legacy mappings for the specified DMO.
    * 
    * @param    schema
    *           The schema to which the DMO belongs.
    * @param    dmoClass
    *           The MDO implementation class.
    */
   static void mapPermanentDMO(String schema, Class<?> dmoClass)
   {
      // - mapping is kept by schema
      // - permamentDBs are kept in a global map
      synchronized (permanentDBs)
      {
         PermanentTableMapper mapper = locatePerm(schema);
         if (mapper == null)
         {
            mapper = new PermanentTableMapper(schema);
         }
         
         mapper.mapClass(dmoClass);
      }
   }

   /**
    * Remove the mappings for the specified schema.
    * 
    * @param    schema
    *           The schema name.
    */
   static void removePermanentSchema(String schema)
   {
      synchronized (permanentDBs)
      {
         permanentDBs.remove(schema);
      }
   }

   /**
    * Remove all the temporary tables for this context.
    */
   static void removeTemporaryTables()
   {
      locateTemp().clear();
   }
   
   /**
    * Add the legacy mappings for the given temporary table.
    * 
    * @param    table
    *           The {@link TempTable} resource.
    */
   static void mapTemporaryTable(TempTable table)
   {
      if (table == null)
      {
         // TODO: this should throw an NPE, but for know leave it like this, to pass regression
         // testing
         return;
      }
      
      locateTemp().mapClass(table);
   }

   /**
    * Remove the mappings for the given temporary table.
    * 
    * @param    table
    *           The {@link TempTable} resource to be removed.
    */
   static void removeTemporaryTable(TempTable table)
   {
      if (table == null)
      {
         // TODO: this is temporary, until BUFFER:TABLE-HANDLE is implemented
         return;
      }
      locateTemp().removeClass(getDMOClass(table));
   }
   
   /**
    * Locate the context-local {@link TempTableMapper}.
    * 
    * @return   See above.
    */
   private static TempTableMapper locateTemp()
   {
      return tempDB.get();
   }
   
   /**
    * Locate the {@link PermanentTableMapper} instance for the given schema.
    * 
    * @param    schema
    *           The schema name.
    *           
    * @return   See above.
    */
   private static PermanentTableMapper locatePerm(String schema)
   {
      return permanentDBs.get(schema);
   }

   /**
    * Get the legacy table name associated with the given DMO implemenation class.
    * 
    * @param    dmoClass
    *           The DMO implementation class.
    *           
    * @return   The legacy table name.
    */
   protected String legacyName(Class<?> dmoClass)
   {
      LegacyTableInfo tableInfo = j2p.get(dmoClass);
      
      return tableInfo.legacyName;
   }
   
   /**
    * Get the legacy field name for given property defined by the specified DMO.
    * 
    * @param    dmoClass
    *           The DMO implementation class.
    * @param    property
    *           The DMO property name.
    *           
    * @return   The legacy field name.
    */
   protected String legacyFieldName(Class<?> dmoClass, String property)
   {
      LegacyTableInfo tableInfo = j2p.get(dmoClass);
      LegacyFieldInfo fieldInfo = tableInfo.j2p.get(property);
      
      return fieldInfo.legacyName;
   }
   
   /**
    * Map the specified DMO implementation class using the specified <code>legacyKey</code>.
    * <p>
    * Before calling this method, make sure the passed DMO has a {@link LegacyTable} annotation.
    * 
    * @param    legacyKey
    *           The legacy key to identify this DMO implementation class.
    * @param    dmoClass
    *           The DMO implementation class.
    */
   protected void mapClass(K legacyKey, Class<?> dmoClass)
   {
      if (j2p.containsKey(dmoClass))
      {
         // already mapped
         return;
      }
      
      LegacyTableInfo tableInfo = new LegacyTableInfo(dmoClass);
      
      p2j.put(legacyKey, tableInfo);
      j2p.put(tableInfo.dmoClass, tableInfo);
   }
   
   /**
    * Remove the mappings for the specified DMO implementation class.
    * 
    * @param    dmoClass
    *           The DMO implementation class.
    */
   protected void removeClass(Class<?> dmoClass)
   {
      LegacyTableInfo info = j2p.get(dmoClass);
      
      if (info == null)
      {
         // ignore if not found
         return;
      }
      
      j2p.remove(info.dmoClass);
      p2j.remove(info.legacyName);
   }
   
   /**
    * Get the DMO class for the legacy table identified with the specified <code>legacyKey</code>.
    * 
    * @param    legacyKey
    *           The key used to identify the {@link LegacyTableInfo}.
    * 
    * @return   The DMO class.
    */
   protected Class<?> getDMOClass(K legacyKey)
   {
      LegacyTableInfo tableInfo = p2j.get(legacyKey);
      
      return tableInfo.dmoClass;
   }

   /**
    * Get the property name for the specified field. The legacy table info is determined using
    * the specified <code>legacyKey</code>.
    * 
    * @param    legacyKey
    *           The key used to identify the {@link LegacyTableInfo}.
    * @param    field
    *           The legacy field name.
    *           
    * @return   The DMO property name associated with the legacy field.
    */
   protected String getPropertyName(K legacyKey, String field)
   {
      LegacyTableInfo tableInfo = p2j.get(legacyKey);
      LegacyFieldInfo fieldInfo = tableInfo.p2j.get(field.toLowerCase());
      
      return fieldInfo.javaName;
   }
   
   /**
    * Clear all the mappings.
    */
   protected void clear()
   {
      j2p.clear();
      p2j.clear();
   }
   
   /**
    * Mapper for permanent tables.
    */
   private static class PermanentTableMapper
   extends TableMapper<String>
   {
      /**
       * Default c'tor.
       * 
       * @param    schema
       *           The schema name.
       */
      public PermanentTableMapper(String schema)
      {
         super(schema);
      }

      /**
       * Register the mappings for the specified permanent DMO.
       * 
       * @param    dmoClass
       *           The DMO implementation class.
       * 
       * @throws   NullPointerException
       *           If the DMO class does not have a {@link LegacyTable} annotation.
       */
      protected void mapClass(Class<?> dmoClass)
      {
         if (!dmoClass.isAnnotationPresent(LegacyTable.class))
         {
            final String msg = "DMO class %s must have a LegacyTable annotation!"; 
            throw new NullPointerException(String.format(msg, dmoClass.getName()));
         }
         
         LegacyTable table = dmoClass.getAnnotation(LegacyTable.class);
         
         mapClass(table.name().toLowerCase(), dmoClass);
      }
   }
   
   /**
    * Mapper for temporary tables.
    */
   private static class TempTableMapper
   extends TableMapper<Integer>
   {
      /**
       * Default c'tor.
       */
      public TempTableMapper()
      {
         super(DatabaseManager.TEMP_TABLE_SCHEMA);
      }

      /**
       * Register the mappings for the specified temp table.
       * 
       * @param    tt
       *           The temp-table which needs to be mapped.
       *           
       * @throws   NullPointerException
       *           If the associated DMO class does not have a {@link LegacyTable} annotation.
       */
      protected void mapClass(TempTable tt)
      {
         Class<?> dmoClass = getDMOClass(tt);
         
         if (!dmoClass.isAnnotationPresent(LegacyTable.class))
         {
            final String msg = "DMO class %s must have a LegacyTable annotation!"; 
            throw new NullPointerException(String.format(msg, dmoClass.getName()));
         }

         Integer key = tt.getUniqueID().toJavaIntegerType();
         
         mapClass(key, dmoClass);
      }
   }
   
   /**
    * A container with the legacy info for a table.
    */
   private static class LegacyTableInfo
   {
      /** Field info kept by legacy name in progress. */ 
      private final Map<String, LegacyFieldInfo> p2j = new HashMap<String, LegacyFieldInfo>();
      
      /** Field info kept by associated DMO property name. */ 
      private final Map<String, LegacyFieldInfo> j2p = new HashMap<String, LegacyFieldInfo>();
      
      /** The legacy table name. */
      private final String legacyName;
      
      /** The DMO class. */
      private final Class<?> dmoClass;
      
      /**
       * Initialize the legacy info for a table by reading the {@link LegacyTable} and 
       * {@link LegacyField} annotation from the DMO implementation class.
       * 
       * @param    dmoClass
       *           The DMO class from where the annotations will be read.
       * 
       * @see      #loadFields(Class)
       */
      public LegacyTableInfo(Class<?> dmoClass)
      {
         LegacyTable ltable = dmoClass.getAnnotation(LegacyTable.class);
         
         this.legacyName = ltable.name();
         this.dmoClass = dmoClass;
         
         loadFields(dmoClass);
         
         Class<?>[] classMembers = dmoClass.getDeclaredClasses();
         for (Class<?> cls : classMembers)
         {
            loadFields(cls);
         }
      }
      
      /**
       * Load all fields from the specified class and add their mappings.
       * 
       * @param    cls
       *           The DMO class or a composite class with the extent fields.
       * 
       * @throws   NullPointerException
       *           If a DMO property does not have a {@link LegacyField} annotation.
       */
      private void loadFields(Class<?> cls)
      {
         Field[] fields = cls.getDeclaredFields();
         for (Field field : fields)
         {
            LegacyField lfield = field.getAnnotation(LegacyField.class);
            
            if (lfield == null)
            {
               final String msg = "Property %s for DMO %s must have a LegacyField annotation!"; 
               throw new NullPointerException(
                  String.format(msg, field.getName(), dmoClass.getName()));
            }

            String pname = lfield.name();
            String jname = field.getName();
            
            LegacyFieldInfo fieldInfo = new LegacyFieldInfo(pname, jname);
            
            p2j.put(pname.toLowerCase(), fieldInfo);
            j2p.put(jname, fieldInfo);
         }
      }
   }
   
   /**
    * A container with a field's legacy and DMO property name.
    */
   private static class LegacyFieldInfo
   {
      /** The legacy name of this field. */
      private final String legacyName;
      
      /** The associated DMO property name. */
      private final String javaName;
      
      /**
       * Create a new field information container, holding the legacy and converted names.
       * 
       * @param    legacyName
       *           The field's legacy name.
       * @param    javaName
       *           The field's associated DMO property name.
       */
      public LegacyFieldInfo(String legacyName, String javaName)
      {
         this.legacyName = legacyName;
         this.javaName = javaName;
      }
   }
}
