/*
** Module   : JavaTokenTypes.java
** Abstract : represents all constructs of the Java language in a form that
**            can be represented in an AST
**
** Copyright (c) 2005-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 GES 20050323   @20488 First version, with support for comments, 
**                           the 5 types of Java tokens: identifier,
**                           keyword, literal, separator and operator.
**                           Additional artificial tokens are added
**                           where it is known that additional tokens
**                           are needed.  This is primarily in the area
**                           of expressions and control flow.
** 002 GES 20050421   @20835 Minor modifications to better support
**                           object references and method definitions.
** 003 GES 20050430   @20949 Changes to support blocks, labels and control
**                           flow processing.
** 004 GES 20050501   @20957 Changes to support variable definitions and
**                           variable references.
** 005 GES 20050505   @21119 Modified array subscripting support.
** 006 GES 20050530   @21450 Modifications to support new pattern/expr
**                           engine changes AND to support the new
**                           data type wrappers.
** 007 GES 20050614   @21493 Added some flow control constructs like
**                           ELSE_IF and OR_ELSE_IF.  Also added a BOGUS
**                           token as a valid node that can be a root
**                           node for other ASTs but which doesn't ever
**                           get emitted to Java source.
** 008 ECF 20050629   @21599 Added token types for enhanced javadoc
**                           support. These are DOC_PARAGRAPH,
**                           DOC_TEXTRUN, DOC_HTML, DOCTAG_LINK,
**                           DOCTAG_PARAM, and DOCTAG_RETURN.
** 009 ECF 20050706   @21671 Added MEMBER token. This is used for direct
**                           member access, as in "this.memberVariable".
** 010 ECF 20050721   @21771 Added CS_* token types. These are used to
**                           create stuctural anchor points for different
**                           sections within the Java class AST. It is to
**                           these anchors which we attach members of the
**                           class: method definitions, inner classes,
**                           variable declarations, etc.
** 011 GES 20050818   @22185 Added DO_WHILE construct.
** 012 ECF 20050922   @22884 Added METHOD_DECL constant. This is used to
**                           declare interface or abstract methods which
**                           have no body.
** 013 ECF 20060111   @23869 Added ANON_CTOR. Identifies an anonymous,
**                           inner class constructor.
** 014 GES 20070221   @32192 Added PLACEHOLDER to be an even more silent
**                           node than BOGUS.
** 015 GES 20080701   @39055 Added Java 5 keywords and a static_import
**                           type.
** 016 GES 20080814   @39432 Added SKIP and INDENT_GRP for better output
**                           control.
** 017 CA  20131015          Added constant to identify a Java annoation.
*/

package com.goldencode.p2j.uast;

/**
 * Provides a complete set of token types sufficient to represent any Java
 * program in an AST form.  All Java programs are comprised of whitespace,
 * comments and tokens.  Whitespace is not represented in an AST.  Comments
 * and tokens must be handled.  All valid tokens can be separated into one of
 * 5 possible categories: identifier, keyword, literal, separator and 
 * operator.
 * <p>
 * Most separators are 'syntactic sugar' designed to allow easy parsing but
 * not useful in the intermediate form (AST). Such constructs are not
 * supported in this interface.  When the AST is output to a Java compilation
 * unit, they will be present. The following are not directly supported:
 * <p>
 * <pre>
 * {
 * }
 * ]
 * ,
 * ;
 * .
 * )
 * </pre>
 * <p>
 * The following operators are rarely used and are not supported at this
 * time:
 * <p>
 * <pre>
 * *=
 * /=
 * &=
 * |=
 * ^=
 * %=
 * &lt;&lt;=
 * &gt;&gt;=
 * &gt;&gt;&gt;=
 * </pre>
 * <p>
 * Some number of 'artificial' tokens are provided to allow the proper
 * structuring of trees but for which no Java source code would ever be
 * emitted.  Examples include {@link #THEN} and {@link #BOGUS}.
 *
 * @author    GES
 * @version   1.0.6
 */
public interface JavaTokenTypes
{
   /** The root node of any Java compilation unit. */
   public static final int COMPILE_UNIT = 0;
   
   /** The 'double slash' style comment. */
   public static final int SLASH_COMMENT = 1;
   
   /** The 'slash star' style comment. */
   public static final int STAR_COMMENT = 2;
   
   /** The 'JavaDoc' style comment. */
   public static final int DOC_COMMENT = 3;
   
   /**
    * Any valid identifier (that is not a keyword, boolean literal or the null
    * literal).
    */
   public static final int SYMBOL = 4;
   
   /** The reserved keyword <code>abstract</code>. */
   public static final int KW_ABSTRACT = 5;
   
   /** The reserved keyword <code>assert</code>. */
   public static final int KW_ASSERT = 6;
   
   /** The reserved keyword <code>boolean</code>. */
   public static final int KW_BOOLEAN = 7;
   
   /** The reserved keyword <code>break</code>. */
   public static final int KW_BREAK = 8;
   
   /** The reserved keyword <code>byte</code>. */
   public static final int KW_BYTE = 9;
   
   /** The reserved keyword <code>case</code>. */
   public static final int KW_CASE = 10;
   
   /** The reserved keyword <code>catch</code>. */
   public static final int KW_CATCH = 11;
   
   /** The reserved keyword <code>char</code>. */
   public static final int KW_CHAR = 12;
   
   /** The reserved keyword <code>class</code>. */
   public static final int KW_CLASS = 13;
   
   /** The reserved but unused keyword <code>const</code>. */
   public static final int KW_CONST = 14;
   
   /** The reserved keyword <code>continue</code>. */
   public static final int KW_CONTINUE = 15;
   
   /** The reserved keyword <code>default</code>. */
   public static final int KW_DEFAULT = 16;
   
   /** The reserved keyword <code>do</code>. */
   public static final int KW_DO = 17;
   
   /** The reserved keyword <code>double</code>. */
   public static final int KW_DOUBLE = 18;
   
   /** The reserved keyword <code>else</code>. */
   public static final int KW_ELSE = 19;
   
   /** The reserved keyword <code>enum</code>. */
   public static final int KW_ENUM = 20;
   
   /** The reserved keyword <code>extends</code>. */
   public static final int KW_EXTENDS = 21;
   
   /** The reserved keyword <code>final</code>. */
   public static final int KW_FINAL = 22;
   
   /** The reserved keyword <code>finally</code>. */
   public static final int KW_FINALLY = 23;
   
   /** The reserved keyword <code>float</code>. */
   public static final int KW_FLOAT = 24;
   
   /** The reserved keyword <code>for</code>. */
   public static final int KW_FOR = 25;
   
   /** The reserved but unused keyword <code>goto</code>. */
   public static final int KW_GOTO = 26;
   
   /** The reserved keyword <code>if</code>. */
   public static final int KW_IF = 27;
   
   /** The reserved keyword <code>implements</code>. */
   public static final int KW_IMPLEMENTS = 28;
   
   /** The reserved keyword <code>import</code>. */
   public static final int KW_IMPORT = 29;
   
   /** The reserved keyword <code>instanceof</code>. */
   public static final int KW_INSTANCEOF = 30;
   
   /** The reserved keyword <code>int</code>. */
   public static final int KW_INT = 31;
   
   /** The reserved keyword <code>interface</code>. */
   public static final int KW_INTERFACE = 32;
   
   /** The reserved keyword <code>long</code>. */
   public static final int KW_LONG = 33;
   
   /** The reserved keyword <code>native</code>. */
   public static final int KW_NATIVE = 34;
   
   /** The reserved keyword <code>new</code>. */
   public static final int KW_NEW = 35;
   
   /** The reserved keyword <code>package</code>. */
   public static final int KW_PACKAGE = 36;
   
   /** The reserved keyword <code>private</code>. */
   public static final int KW_PRIVATE = 37;
   
   /** The reserved keyword <code>protected</code>. */
   public static final int KW_PROTECTED = 38;
   
   /** The reserved keyword <code>public</code>. */
   public static final int KW_PUBLIC = 39;
   
   /** The reserved keyword <code>return</code>. */
   public static final int KW_RETURN = 40;
   
   /** The reserved keyword <code>short</code>. */
   public static final int KW_SHORT = 41;
   
   /** The reserved keyword <code>static</code>. */
   public static final int KW_STATIC = 42;
   
   /** The reserved keyword <code>strictfp</code>. */
   public static final int KW_STRICTFP = 43;
   
   /** The reserved keyword <code>super</code>. */
   public static final int KW_SUPER = 44;
   
   /** The reserved keyword <code>switch</code>. */
   public static final int KW_SWITCH = 45;
   
   /** The reserved keyword <code>synchronized</code>. */
   public static final int KW_SYNCHRONIZED = 46;
   
   /** The reserved keyword <code>this</code>. */
   public static final int KW_THIS = 47;
   
   /** The reserved keyword <code>throw</code>. */
   public static final int KW_THROW = 48;
   
   /** The reserved keyword <code>throws</code>. */
   public static final int KW_THROWS = 49;
   
   /** The reserved keyword <code>transient</code>. */
   public static final int KW_TRANSIENT = 50;
   
   /** The reserved keyword <code>try</code>. */
   public static final int KW_TRY = 51;
   
   /** The reserved keyword <code>void</code>. */
   public static final int KW_VOID = 52;
   
   /** The reserved keyword <code>volatile</code>. */
   public static final int KW_VOLATILE = 53;
   
   /** The reserved keyword <code>while</code>. */
   public static final int KW_WHILE = 54;
   
   /** Represents the boolean <code>true</code> literal. */
   public static final int BOOL_TRUE = 55;         
   
   /** Represents the boolean <code>false</code> literal. */
   public static final int BOOL_FALSE = 56;         
   
   /** Represents the <code>null</code> literal. */
   public static final int NULL_LITERAL = 57;      

   /** Represents an integral literal. */
   public static final int NUM_LITERAL = 100;      

   /** Represents a floating point literal. */
   public static final int DEC_LITERAL = 101;      
   
   /** Represents a character literal. */
   public static final int CHAR_LITERAL = 102;      
      
   /** Represents a string literal. */
   public static final int STRING = 103;
   
   /** The left or open parenthesis separator '('. */
   public static final int LPARENS = 104;

   /** Roots Java block contents with a label and other constructs. */
   public static final int BLOCK_DEF = 105;
   
   /** Represents a block of code '{  }' delimited by curly braces. */
   public static final int BLOCK = 106;
   
   /** Represents a block's label. */
   public static final int LABEL_DEF = 107;
   
   /** Represents a reference to a previously defined block label. */
   public static final int LABEL = 108;
   
   /** Allows the primary branch of an 'if' to be rooted properly. */
   public static final int THEN = 109;
   
   /** Allows 'else if' to be generated without nesting. */
   public static final int ELSE_IF = 110;
   
   /**
    * Allows 'else if' to be aggregated into a single logical test without 
    * nesting.
    */
   public static final int OR_ELSE_IF = 111;
   
   /** Allows 'do { } while ();' to be generated cleanly. */
   public static final int DO_WHILE = 112;
   
   /** Represents an expression. */
   public static final int EXPRESSION = 113;
   
   /** Represents a definition of an instance of a specific object. */
   public static final int REFERENCE_DEF = 114;   
   
   /** Represents a reference to an instance of a specific object. */
   public static final int REFERENCE = 115;
   
   /** Represents a reference to a class. */
   public static final int CLASS_REFERENCE = 116;   
   
   /** 
    * Represents an array initializer block '{ , , , }' which is delimited by
    * curly braces.
    */
   public static final int INITIALIZER = 117;
   
   /** The array subscript '[' separator. */
   public static final int LBRACKET = 118;
   
   /** Represents a static method call. */
   public static final int STATIC_METHOD_CALL = 119;
   
   /** Represents a special static method call which is a constructor. */
   public static final int CONSTRUCTOR = 120;
   
   /** Represents an instance method call. */
   public static final int METHOD_CALL = 121;
   
   /** Represents an instance method definition. */
   public static final int METHOD_DEF = 122;
   
   /** Represents an instance method declaration (signature only, no body) */
   public static final int METHOD_DECL = 123;
   
   /** Represents a member reference */
   public static final int MEMBER = 124;
   
   /** Constructor for an anonymous inner class;  text is superclass name */
   public static final int ANON_CTOR = 125;
   
   /** Represents a static import statement. */
   public static final int STATIC_IMPORT = 126;
   
   /** Represents an annotation. */
   public static final int ANNOTATION = 127;      

   /** Represents a cast operator '( )'. */
   public static final int CAST = 700;   
   
   /** The assignment operator '='. */
   public static final int ASSIGN = 701;

   /** The less-than operator '&lt;'. */
   public static final int LT = 702;
   
   /** The less-than or equal-to operator '&lt;='. */
   public static final int LTE = 703;
   
   /** The greater-than operator '&gt;'. */
   public static final int GT = 704;
      
   /** The greater-than or equal-to operator '&gt;='. */
   public static final int GTE = 705;

   /** The bitwise complement operator '~'. */
   public static final int COMPLEMENT = 706;
   
   /** The ternary if else operator '&#63; &#58;'. */
   public static final int TERN_IF_ELSE = 707;
   
   /** The equivalance operator '=='. */
   public static final int EQUALS = 708;
   
   /** The not equivalant operator '&#33;='. */
   public static final int NOT_EQ = 709;
   
   /** The increment operator '++'. */
   public static final int INCREMENT = 710;

   /** The decrement operator '--'. */
   public static final int DECREMENT = 711;

   /** The logical not operator '&#33;'. */
   public static final int LOGICAL_NOT = 712;
   
   /** The logical and operator '&&'. */
   public static final int LOGICAL_AND = 713;
   
   /** The logical or operator '||'. */
   public static final int LOGICAL_OR = 714;
   
   /** The bitwise and operator '&'. */
   public static final int BITWISE_AND = 715;
   
   /** The bitwise or operator '|'. */
   public static final int BITWISE_OR = 716;
   
   /** The bitwise exclusive or operator '^'. */
   public static final int BITWISE_XOR = 717;
   
   /** The addition operator '+'. */
   public static final int PLUS = 718;
   
   /** The subtraction operator '-'. */
   public static final int MINUS = 719;
   
   /** The multiplication operator '*'. */
   public static final int MULTIPLY = 720;
   
   /** The division operator '/'. */
   public static final int DIVIDE = 721;
   
   /** The modulo operator '%'. */
   public static final int MODULO = 722;
   
   /** The unary plus operator '+'. */
   public static final int UN_PLUS = 723;
   
   /** The unary minus (negation) operator '-'. */
   public static final int UN_MINUS = 724;
   
   /** The left shift operator '&lt;&lt;'. */
   public static final int LSHIFT = 725;
   
   /** The right shift operator '&gt;&gt;'. */
   public static final int RSHIFT = 726;
   
   /** The zero fill right shift operator '&gt;&gt;&gt;'. */
   public static final int ZERO_RSHIFT = 727;
   
   /** The add to and assign operator '+='. */
   public static final int PLUS_ASSIGN = 728;
   
   /** The subtract from and assign operator '-='. */
   public static final int MINUS_ASSIGN = 729;
   
   /**
    * JavaDoc paragraph of text; all textual content is rooted at a
    * paragraph. Has text run and other tags as children.
    */
   public static final int DOC_PARAGRAPH = 800;
   
   /** Run of text within a JavaDoc comment; used within paragraph or tags */
   public static final int DOC_TEXTRUN = 801;
   
   /**
    * Tag which can parent a text run. Token text is the HTML element name.
    * All annotations are translated to HTML attributes, where annotation
    * key becomes attribute key, annotation value becomes attribute value.
    */
   public static final int DOC_HTML = 802;
   
   /** Beginning of JavaDoc tag range */
   public static final int BEGIN_DOCTAGS = 810;
   
   /**
    * JavaDoc link tag; embedded within a paragraph or other tag. Text is
    * not used. Has optional 'package', 'class', 'member', and 'label'
    * annotations, at least one of which should be present to form a valid
    * link tag.
    */
   public static final int DOCTAG_LINK = 811;
   
   /** JavaDoc param tag; text is param name, has text run and tag children */
   public static final int DOCTAG_PARAM = 812;
   
   /** JavaDoc return tag; has text run and tag children */
   public static final int DOCTAG_RETURN = 813;
   
   /** End of JavaDoc tag range */
   public static final int END_DOCTAGS = 850;
   
   /** Beginning of (exclusive) range of class section anchor point types */
   public static final int CS_BEGIN = 900;
   
   /** Anchor point for all static final class constants */
   public static final int CS_CONSTANTS = 901;
   
   /** Anchor point for all static variables */
   public static final int CS_STATIC_VARS = 902;
   
   /** Anchor point for all static initializers */
   public static final int CS_STATIC_INITS = 903;
   
   /** Anchor point for all instance variables */
   public static final int CS_INSTANCE_VARS = 904;
   
   /** Anchor point for all constructors */
   public static final int CS_CONSTRUCTORS = 905;
   
   /** Anchor point for all static methods */
   public static final int CS_STATIC_METHODS= 906;
   
   /** Anchor point for all instance methods */
   public static final int CS_INSTANCE_METHODS= 907;
   
   /** Anchor point for all inner classes */
   public static final int CS_INNER_CLASSES = 908;
   
   /** End of (exclusive) range of class section anchor point types */
   public static final int CS_END = 909;
   
   /**
    * Inserts a blank line and then indents to the same level for the next
    * output.
    */
   public static final int SKIP = 996;
   
   /**
    * Aggregates nodes and increments the indent level before the first
    * child node is output and decrements the indent level after the last
    * child node is output. This is similar to the BLOCK but no curly braces
    * are output.
    */
   public static final int INDENT_GRP = 997;
   
   /**
    * A valid token that can be used to root other tokens but which is never
    * emitted as Java source code, nor does it have any impact (positive or
    * negative) on line ends or indentation.
    */
   public static final int PLACEHOLDER = 998;
   
   /**
    * A valid token that can be used to root other tokens and which has the
    * properties of a BLOCK in regards to line ends and indentation but
    * without emitting any corresponding curly braces or other explicit 
    * artifacts.
    */
   public static final int BOGUS = 999;
   
   /**
    * Represents an unrecognized source construct (this provides a 
    * placeholder to maintain the tree structure).
    */
   public static final int UNRECOGNIZED = 1000;
}
